@@ -1,9 +1,9 @@
-use safe_en::{table::Entries, Database};
+use safe_en::Database;
 use std::path::Path;
 
 fn main() {
     //Check if test.sfe file exists
-    if Path::new("./examples/db.sfe").exists() {
+    if Path::new("./examples/db.sfn").exists() {
         let mut db = Database::load("./examples/db.sfn").unwrap();
         println!("Db named '{}' loaded", db.get_name());
         let table = db.table("users").unwrap();
@@ -26,7 +26,7 @@ fn main() {
                 |x| x.row("name").is("Ahmet".to_string()),
                 vec![safe_en::table::Entry {
                     key: "name".to_string(),
-                    value: "Ahmetcan".into(),
+                    value: "Ahmetcanq".into(),
                 }],
             )
             .unwrap();
@@ -37,6 +37,8 @@ fn main() {
 
             println!("")
         }
+
+        println!("{}", table);
     } else {
         panic!("Db file not found! run save_big_data.rs to create it.");
     }
