@@ -1,5 +1,4 @@
-use core::panic;
-use std::{fmt::Display, ops::Index};
+use core::{fmt::Display, ops::Index};
 
 /// Rust types to be used in the table
 #[derive(Clone, Debug, PartialEq)]
@@ -25,7 +24,7 @@ pub enum TypeDefs {
 }
 
 impl Display for TypeDefs {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         match self {
             TypeDefs::String => write!(f, "String"),
             TypeDefs::Char => write!(f, "Char"),
@@ -96,11 +95,11 @@ pub enum Types {
     /// F64 type
     F64(f64),
     /// Array type
-    Array(Vec<Types>),
+    Array(Vec<SafeType>),
 }
 
 impl Display for Types {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         match self {
             Types::String(e) => format!("\"{}\"", e).fmt(f),
             Types::Char(e) => format!("'{}'", e).fmt(f),
@@ -112,13 +111,49 @@ impl Display for Types {
             Types::F64(e) => format!("{}", e).fmt(f),
             Types::Array(e) => format!(
                 "[{}]",
-                e.iter().map(|x| format!("{}", x)).collect::<String>()
+                e.iter()
+                    .map(|x| format!("{}", x.get_type_name()))
+                    .collect::<String>()
             )
             .fmt(f),
         }
     }
 }
 
+///REP
+#[derive(Clone, Debug, PartialEq)]
+pub struct SafeType {
+    ///REP
+    pub type_id: TypeDefs,
+    ///REP
+    pub rtype: Types,
+}
+
+impl SafeType {
+    ///REP
+    pub fn get<T>(&self) -> T
+    where
+        T: core::convert::From<Types>,
+    {
+        Into::into(self.rtype.clone())
+    }
+
+    ///REP
+    pub fn get_type(&self) -> Types {
+        self.rtype.clone()
+    }
+
+    ///REP
+    pub fn get_type_name(&self) -> TypeDefs {
+        self.type_id.clone()
+    }
+
+    ///REP
+    pub fn build(rtype: Types, type_id: TypeDefs) -> SafeType {
+        SafeType { type_id, rtype }
+    }
+}
+
 impl Types {
     // Get defined type
     pub(crate) fn get_defination(&self) -> TypeDefs {
@@ -131,7 +166,154 @@ impl Types {
             Types::Bool(_) => TypeDefs::Bool,
             Types::F32(_) => TypeDefs::F32,
             Types::F64(_) => TypeDefs::F64,
-            Types::Array(e) => TypeDefs::Array(Box::new(e[0].get_defination())),
+            Types::Array(e) => panic!(),
+        }
+    }
+
+    /// Returns true if type is string
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let s = Types::String("Hello".to_string());
+    /// assert_eq!(s.is_string(), true);
+    /// ```
+    pub fn is_string(&self) -> bool {
+        match self {
+            Types::String(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is char
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::Char('a');
+    /// assert_eq!(t.is_char(), true);
+    /// ```
+    pub fn is_char(&self) -> bool {
+        match self {
+            Types::Char(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is i8
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::I8(1);
+    /// assert_eq!(t.is_i8(), true);
+    /// ```
+    pub fn is_i8(&self) -> bool {
+        match self {
+            Types::I8(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is i64
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::I64(1);
+    /// assert_eq!(t.is_i64(), true);
+    /// ```
+    pub fn is_i64(&self) -> bool {
+        match self {
+            Types::I64(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is u64
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::U64(1);
+    /// assert_eq!(t.is_u64(), true);
+    /// ```
+    pub fn is_u64(&self) -> bool {
+        match self {
+            Types::U64(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is bool
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::Bool(true);
+    /// assert_eq!(t.is_bool(), true);
+    /// ```
+    pub fn is_bool(&self) -> bool {
+        match self {
+            Types::Bool(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is f32
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::F32(1.0);
+    /// assert_eq!(t.is_f32(), true);
+    /// ```
+    pub fn is_f32(&self) -> bool {
+        match self {
+            Types::F32(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is f64
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::F64(1.0);
+    /// assert_eq!(t.is_f64(), true);
+    /// ```
+    pub fn is_f64(&self) -> bool {
+        match self {
+            Types::F64(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns true if type is array
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::Array(vec![Types::I8(1)]);
+    /// assert_eq!(t.is_array(), true);
+    /// ```
+    pub fn is_array(&self) -> bool {
+        match self {
+            Types::Array(_) => true,
+            _ => false,
+        }
+    }
+
+    /// Returns the type of array
+    /// # Example
+    /// ```
+    /// use safe_en::table::Types;
+    /// let t = Types::Array(vec![Types::I8(1)]);
+    /// assert_eq!(t.get_array_type(), Some(&vec![Types::I8(1)]));
+    /// ```
+    //pub fn _get_array_type(&self) -> Option<&Vec<Types>> {
+    //    match self {
+    //        Types::Array(e) => Some(e),
+    //        _ => None,
+    //    }
+    //}
+
+    fn is_empty_array(&self) -> bool {
+        match self {
+            Types::Array(e) => e.is_empty(),
+            _ => false,
         }
     }
 
@@ -200,6 +382,60 @@ impl Types {
     }
 }
 
+impl Into<SafeType> for &str {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::String(self.to_string()), TypeDefs::String)
+    }
+}
+
+impl Into<SafeType> for String {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::String(self), TypeDefs::String)
+    }
+}
+
+impl Into<SafeType> for char {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::Char(self), TypeDefs::Char)
+    }
+}
+
+impl Into<SafeType> for i8 {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::I8(self), TypeDefs::I8)
+    }
+}
+
+impl Into<SafeType> for i64 {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::I64(self), TypeDefs::I64)
+    }
+}
+
+impl Into<SafeType> for u64 {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::U64(self), TypeDefs::U64)
+    }
+}
+
+impl Into<SafeType> for bool {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::Bool(self), TypeDefs::Bool)
+    }
+}
+
+impl Into<SafeType> for f32 {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::F32(self), TypeDefs::F32)
+    }
+}
+
+impl Into<SafeType> for f64 {
+    fn into(self) -> SafeType {
+        SafeType::build(Types::F64(self), TypeDefs::F64)
+    }
+}
+
 impl Into<Types> for &str {
     fn into(self) -> Types {
         Types::String(self.to_string())
@@ -258,10 +494,8 @@ impl Into<Types> for Vec<String> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::String(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::String(c), TypeDefs::String))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -270,10 +504,8 @@ impl Into<Types> for Vec<char> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::Char(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::Char(c), TypeDefs::Char))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -282,10 +514,8 @@ impl Into<Types> for Vec<i8> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::I8(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::I8(c), TypeDefs::I8))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -294,10 +524,8 @@ impl Into<Types> for Vec<i64> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::I64(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::I64(c), TypeDefs::I64))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -306,10 +534,8 @@ impl Into<Types> for Vec<u64> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::U64(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::U64(c), TypeDefs::U64))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -318,10 +544,8 @@ impl Into<Types> for Vec<bool> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::Bool(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::Bool(c), TypeDefs::Bool))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -330,10 +554,8 @@ impl Into<Types> for Vec<f32> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::F32(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::F32(c), TypeDefs::F32))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
@@ -342,15 +564,117 @@ impl Into<Types> for Vec<f64> {
     fn into(self) -> Types {
         Types::Array(
             self.into_iter()
-                .map(|c| Types::F64(c))
-                .collect::<Vec<Types>>()
-                .into_iter()
-                .collect::<Vec<Types>>(),
+                .map(|c| SafeType::build(Types::F64(c), TypeDefs::F64))
+                .collect::<Vec<SafeType>>(),
         )
     }
 }
 
-impl Into<Types> for Vec<Types> {
+impl Into<SafeType> for Vec<String> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::String(c), TypeDefs::String))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::String,
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<char> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::Char(c), TypeDefs::Char))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::Char,
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<i8> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::I8(c), TypeDefs::I8))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::I8,
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<i64> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::I64(c), TypeDefs::I64))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::Array(Box::new(TypeDefs::I64)),
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<u64> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::U64(c), TypeDefs::U64))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::Array(Box::new(TypeDefs::U64)),
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<bool> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::Bool(c), TypeDefs::Bool))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::Bool,
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<f32> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::F32(c), TypeDefs::F32))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::Array(Box::new(TypeDefs::F32)),
+        )
+    }
+}
+
+impl Into<SafeType> for Vec<f64> {
+    fn into(self) -> SafeType {
+        SafeType::build(
+            Types::Array(
+                self.into_iter()
+                    .map(|c| SafeType::build(Types::F64(c), TypeDefs::F64))
+                    .collect::<Vec<SafeType>>(),
+            ),
+            TypeDefs::Array(Box::new(TypeDefs::F64)),
+        )
+    }
+}
+
+impl Into<Types> for Vec<SafeType> {
     fn into(self) -> Types {
         Types::Array(self)
     }
@@ -431,13 +755,7 @@ impl From<Types> for f64 {
 impl From<Types> for Vec<String> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::String(y) => y,
-                    _ => panic!("Not a String type"),
-                })
-                .collect::<Vec<String>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<String>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -446,13 +764,7 @@ impl From<Types> for Vec<String> {
 impl From<Types> for Vec<char> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::Char(y) => y,
-                    _ => panic!("Not a char type"),
-                })
-                .collect::<Vec<char>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<char>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -461,13 +773,7 @@ impl From<Types> for Vec<char> {
 impl From<Types> for Vec<i8> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::I8(y) => y,
-                    _ => panic!("Not a i8 type"),
-                })
-                .collect::<Vec<i8>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<i8>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -476,13 +782,7 @@ impl From<Types> for Vec<i8> {
 impl From<Types> for Vec<i64> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::I64(y) => y,
-                    _ => panic!("Not a i64 type"),
-                })
-                .collect::<Vec<i64>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<i64>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -491,13 +791,7 @@ impl From<Types> for Vec<i64> {
 impl From<Types> for Vec<u64> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::U64(y) => y,
-                    _ => panic!("Not a u64 type"),
-                })
-                .collect::<Vec<u64>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<u64>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -506,13 +800,7 @@ impl From<Types> for Vec<u64> {
 impl From<Types> for Vec<bool> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::Bool(y) => y,
-                    _ => panic!("Not a bool type"),
-                })
-                .collect::<Vec<bool>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<bool>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -521,13 +809,7 @@ impl From<Types> for Vec<bool> {
 impl From<Types> for Vec<f32> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::F32(y) => y,
-                    _ => panic!("Not a f32 type"),
-                })
-                .collect::<Vec<f32>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<f32>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -536,13 +818,7 @@ impl From<Types> for Vec<f32> {
 impl From<Types> for Vec<f64> {
     fn from(c: Types) -> Self {
         match c {
-            Types::Array(x) => x
-                .into_iter()
-                .map(|f| match f {
-                    Types::F64(y) => y,
-                    _ => panic!("Not a f64 type"),
-                })
-                .collect::<Vec<f64>>(),
+            Types::Array(x) => x.into_iter().map(|f| f.get()).collect::<Vec<f64>>(),
             _ => panic!("Not a vec type"),
         }
     }
@@ -551,21 +827,21 @@ impl From<Types> for Vec<f64> {
 /// Row of table
 /// Key is header of the table
 /// Value is the value of the row
-#[derive(Clone)]
+#[derive(Clone, Debug, PartialEq)]
 pub struct Entry {
     /// Key
     pub key: String,
     /// Value
-    pub value: Types,
+    pub value: SafeType,
 }
 
 impl Entry {
     /// Get the value of the entry
     pub fn get<T>(&self) -> T
     where
-        T: std::convert::From<Types>,
+        T: core::convert::From<SafeType> + std::convert::From<Types>,
     {
-        Into::into(self.value.clone())
+        self.value.get()
     }
 }
 
@@ -574,11 +850,11 @@ impl Entry {
 pub struct Table {
     pub(crate) name: String,
     pub(crate) headers: Vec<TableRow>,
-    pub(crate) columns: Vec<Vec<Types>>,
+    pub(crate) columns: Vec<Vec<SafeType>>,
 }
 
 impl Display for Table {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         let mut lines = String::new();
         let max_row_lengths = if self.columns.len() == 0 {
             self.headers
@@ -626,7 +902,7 @@ impl Display for Table {
 
             for (index, _) in self.headers.iter().enumerate() {
                 let mut row_len = String::new();
-                let rtype = format!(" {}", column[index]);
+                let rtype = format!(" {}", column[index].get_type());
                 for i in 0..max_row_lengths[index] {
                     if i < rtype.len() {
                         row_len += &rtype.chars().nth(i).unwrap_or(' ').to_string();
@@ -669,12 +945,12 @@ pub struct Entries {
 }
 
 impl Display for Entries {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         let mut header_line = String::from("| ");
         let mut value_line = String::from("| ");
         for entry in &self.entries {
             let key_len = entry.key.len();
-            let value_len = format!("{}", entry.value).len();
+            let value_len = format!("{}", entry.value.get_type()).len();
             let line_len = if key_len > value_len {
                 key_len + 1
             } else {
@@ -689,7 +965,7 @@ impl Display for Entries {
                 header_line += "| ";
             }
 
-            value_line += &format!("{}", entry.value);
+            value_line += &format!("{}", entry.value.get_type());
 
             if value_len < line_len {
                 for _ in 0..(line_len - value_len) {
@@ -742,11 +1018,29 @@ impl RowQuery {
     /// ```
     pub fn is<T>(&self, key: T) -> bool
     where
-        T: std::convert::From<Types> + std::cmp::PartialEq,
+        T: Into<SafeType> + PartialEq,
     {
         if let Some(entry) = &self.entry {
-            let key: T = key.into();
-            key == Into::into(entry.value.clone())
+            let f = Into::into(key);
+            f == entry.value
+            /*
+            match &f {
+                Types::String(_) => entry.value.get_type_name() == TypeDefs::String && entry.value == f,
+                Types::Char(_) => entry.value.get_type_name() == TypeDefs::Char && entry.value == f,
+                Types::I8(_) => entry.value.is_i8() && entry.value == f,
+                Types::I64(_) => entry.value.is_i64() && entry.value == f,
+                Types::U64(_) => entry.value.is_u64() && entry.value == f,
+                Types::F32(_) => entry.value.is_f32() && entry.value == f,
+                Types::F64(_) => entry.value.is_f64() && entry.value == f,
+                Types::Bool(_) => entry.value.is_bool() && entry.value == f,
+                Types::Array(e) => {
+                    entry.value.is_array()
+                        && matches!(entry.value.get_array_type(), Some(x) if x == e)
+                        && entry.value == f
+                }
+            }
+            */
+            //key == entry.value.clone().into()
         } else {
             false
         }
@@ -770,7 +1064,7 @@ impl RowQuery {
     /// ```
     pub fn is_it(&self, key: TypeDefs) -> bool {
         if let Some(entry) = &self.entry {
-            key == entry.value.get_defination()
+            key == entry.value.get_type_name()
         } else {
             false
         }
@@ -794,10 +1088,10 @@ impl RowQuery {
     /// ```
     pub fn get_value<T>(&self) -> Option<T>
     where
-        T: std::convert::From<Types>,
+        T: From<Types>,
     {
         if let Some(entry) = &self.entry {
-            Some(Into::into(entry.value.clone()))
+            Some(entry.value.get())
         } else {
             None
         }
@@ -1106,7 +1400,7 @@ impl Table {
                         .iter()
                         .find(|x| x.key == value_entry.key);
                     if let Some(target) = targt {
-                        if target.value.get_defination() == value_entry.value.get_defination() {
+                        if target.value.get_type_name() == value_entry.value.get_type_name() {
                             let header_pos = self
                                 .headers
                                 .iter()
@@ -1117,8 +1411,8 @@ impl Table {
                         } else {
                             errors.push(format!(
                                 "Value type is not equal to header type. Header: {}, Value: {}",
-                                target.value.get_defination(),
-                                value_entry.value.get_defination()
+                                target.value.get_type_name(),
+                                value_entry.value.get_type_name()
                             ));
                             break 'entryloop;
                         }
@@ -1129,10 +1423,10 @@ impl Table {
                 }
             }
         }
-        if errors.len() > 0 {
-            Err(errors)
-        } else {
+        if errors.is_empty() {
             Ok(changed_rows)
+        } else {
+            Err(errors)
         }
     }
 
@@ -1159,7 +1453,7 @@ impl Table {
     /// ## Returns
     /// * [`Result<()>`]
     /// * [`Err<Vec<String>>`] for insert errors
-    pub fn insert(&mut self, rows: Vec<Types>) -> Result<(), Vec<String>> {
+    pub fn insert(&mut self, rows: Vec<SafeType>) -> Result<(), Vec<String>> {
         let mut errors = vec![];
         if rows.len() != self.headers.len() {
             errors.push(format!(
@@ -1174,13 +1468,13 @@ impl Table {
         for i in 0..rows.len() {
             let header = &self.headers[i];
 
-            if header.rtype == rows[i].get_defination() {
+            if header.rtype == rows[i].get_type_name() {
                 _rows.push(rows[i].clone());
             } else {
                 errors.push(format!(
                     "Type mismatch, expected {}, got {} on column {}",
                     header.rtype,
-                    rows[i].get_defination(),
+                    rows[i].get_type_name(),
                     i
                 ));
             }
